# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Test base code"""

import unittest
import tempfile
import os
import shutil
import stardicter.base


class BaseTest(unittest.TestCase):
    writer_class = stardicter.base.StardictWriter

    def skip_net(self):
        if 'SKIP_NET_TESTS' in os.environ:
            raise unittest.SkipTest('Network tests disabled')

    def get_writer(self):
        '''
        Gets prepared writer class.
        '''
        self.skip_net()
        return self.writer_class()

    def test_write(self):
        '''
        Test dictionary writing.
        '''
        writer = self.get_writer()

        directory = tempfile.mkdtemp(prefix='stardicter-tmp')

        # Parse data
        writer.parse()
        # Write standard
        writer.write_dict(directory)
        # Write ascii
        writer.ascii = True
        writer.write_dict(directory)
        # Write notags
        writer.notags = True
        writer.write_dict(directory)

        # Check readme
        self.assertTrue(os.path.exists(os.path.join(directory, 'README')))

        shutil.rmtree(directory)


class BaseObjectTest(unittest.TestCase):
    '''
    Testing of base object.
    '''
    def test_checksum(self):
        '''
        Test checksum generating.
        '''
        writer = stardicter.base.StardictWriter()
        self.assertEqual(writer.checksum, '6d43d48e5163b5a24c5d4dc92deca1f3')

    def changes_testing(self, name):
        '''
        Core for changes testing.
        '''
        backup = stardicter.base.CONFIGFILE
        stardicter.base.CONFIGFILE = name
        writer = stardicter.base.StardictWriter()
        self.assertTrue(writer.was_changed())
        writer.save_checksum()
        self.assertFalse(writer.was_changed())
        stardicter.base.CONFIGFILE = backup

    def test_changes(self):
        '''
        Test changes detection with empty config file.
        '''
        with tempfile.NamedTemporaryFile(delete=True) as temp:
            self.changes_testing(temp.name)

    def test_changes_nofile(self):
        '''
        Test changes detection without config file.
        '''
        temp = tempfile.NamedTemporaryFile(delete=True)
        temp.close()
        self.changes_testing(temp.name)



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Special codecs filter to remove accents"""

from __future__ import unicode_literals
import unicodedata


SIMPLE_MAPS = {
    'ACUTE ACCENT': '\'',
    'NO-BREAK SPACE': ' ',
    'THIN SPACE': ' ',
    'MULTIPLICATION SIGN': 'x',
    'DEGREE SIGN': '<degree>',
    # §
    'SECTION SIGN': '<paragraph>',
    # ÷
    'DIVISION SIGN': '/',
    # „
    'DOUBLE LOW-9 QUOTATION MARK': '"',
    # “
    'LEFT DOUBLE QUOTATION MARK':  '"',
    # –
    'EN DASH': '-',
}


def deaccent(exc):
    '''
    Removes accents on string conversion errors.
    '''
    if not isinstance(exc, UnicodeEncodeError):
        raise TypeError("don't know how to handle {0}".format(exc))
    result = []
    for current in exc.object[exc.start:exc.end]:
        #  print('"{0}" {1}'.format(current, ord(current)))
        if current in ('\x93', '\x94', '\x84'):
            result.append('"')
            continue
        elif current == '\x92':
            result.append('\'')
            continue
        name = unicodedata.name(current)
        if name[:18] == 'LATIN SMALL LETTER':
            result.append(name[19].lower())
        elif name[:20] == 'LATIN CAPITAL LETTER':
            result.append(name[21])
        elif name in SIMPLE_MAPS:
            result.append(SIMPLE_MAPS[name])
        else:
            raise ValueError(
                'Can not convert to ASCII: {0} ({1})'.format(current, name)
            )
    return (''.join(result), exc.end)



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Dictionary word manipulation"""

from __future__ import unicode_literals
from stardicter.utils import reformat, xmlescape

FMT_TYPE = '<span size="larger" color="darkred" weight="bold">{0}</span>\n'
FMT_DETAILS = '<i>{0}</i> '
FMT_TRANSLATE = '<b>{0}</b>'
FMT_NOTE = ' ({0})'
FMT_AUTHOR = ' <small>[{0}]</small>'
FMT_PRONUNCIATION = '[<i>{0}</i>]\n\n'


class Word(object):
    '''
    Class holding single word.
    '''
    def __init__(self, word, translation, wtype='', note='', author='',
                 pronunciation=''):
        self.word = word
        self.translation = translation
        self.wtype = wtype
        self.note = note
        self.author = author
        self.pronunciation = pronunciation

    def reverse(self):
        '''
        Returns copy of a object for reverse direction.
        '''
        return Word(
            self.translation, self.word, self.wtype, self.note, self.author,
            self.pronunciation
        )

    @staticmethod
    def from_slovnik(line):
        '''
        Parses word from format used by https://www.svobodneslovniky.cz/
        '''
        # split it up
        parts = line.split('\t')
        if len(parts) == 5:
            word, translation, wtype, note, author = parts
        elif len(parts) == 6:
            # Extra word
            word, ignore, translation, wtype, note, author = parts
            word += ignore
        elif len(parts) == 1:
            # Missing author, translation, type and note
            word = parts[0]
            translation = ''
            wtype = ''
            author = ''
            note = ''
        elif len(parts) == 2:
            # Missing author, type and note
            word, translation = parts
            wtype = ''
            author = ''
            note = ''
        elif len(parts) == 3:
            # Missing author and note
            word, translation, wtype = parts
            author = ''
            note = ''
        elif len(parts) == 4:
            # Missing author
            word, translation, wtype, note = parts
            author = ''
        else:
            raise ValueError('Invalid input: {0!r}'.format(line))

        return Word(
            word=reformat(word),
            translation=reformat(translation),
            wtype=reformat(wtype),
            note=reformat(note),
            author=reformat(author)
        )

    def format(self):
        '''
        Returns formatted dictionary entry.
        '''
        result = []
        if self.pronunciation != '':
            result.append(
                FMT_PRONUNCIATION.format(xmlescape(self.pronunciation))
            )
        if self.wtype != '':
            result.append(FMT_DETAILS.format(xmlescape(self.wtype)))
        result.append(FMT_TRANSLATE.format(xmlescape(self.translation)))
        if self.note != '':
            result.append(FMT_NOTE.format(xmlescape(self.note)))
        if self.author != '':
            result.append(FMT_AUTHOR.format(xmlescape(self.author)))
        result.append('\n')
        return ''.join(result)



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Convertor for Slovník cizích slov"""

from __future__ import unicode_literals
from stardicter.base import StardictWriter
from stardicter.word import Word

URL = 'http://slovnik-cizich-slov.abz.cz/export.php'


class CzechWriter(StardictWriter):
    url = 'http://slovnik-cizich-slov.abz.cz/'
    name = 'Slovník cizích slov'
    source = 'czech'
    target = 'cizi'
    license = 'CC-BY-3.0'
    bidirectional = False
    download_url = URL
    download_charset = 'iso-8859-2'

    def parse_line(self, line):
        word, pronunciation, explanation = line.split('|')
        return [Word(word, explanation, pronunciation=pronunciation)]

    def get_source_name(self):
        """Name for source file."""
        return 'czech.txt'



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Test for cs"""

import stardicter.czech
from stardicter.test_base import BaseTest


class CzechTest(BaseTest):
    writer_class = stardicter.czech.CzechWriter



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Convertor for GNU/FDL Německo-Český slovník"""

from __future__ import unicode_literals
from stardicter.base import StardictWriter

URL = (
    'http://gnu.nemeckoceskyslovnik.cz/'
    'index.php?id=6&sablona=export&format=zcu'
)


class CzechGermanWriter(StardictWriter):
    url = 'http://www.nemeckoceskyslovnik.cz/'
    name = 'GNU/FDL Německo-Český slovník'
    source = 'german'
    target = 'czech'
    license = 'GFDL-1.1'
    download_url = URL

    def is_data_line(self, line):
        '''
        Checks whether this is line with timestamp.
        '''
        return not line.startswith('# File generated')

    def get_name(self, forward=True):
        if forward:
            return 'GNU/FDL Německo-Český slovník'
        return 'GNU/FDL Česko-Německý slovník'

    def get_source_name(self):
        """Name for source file."""
        return 'de-cs.txt'



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Test for en-cs"""

import io
import os.path

import stardicter.czechenglish
from stardicter.test_base import BaseTest


class CzechEnglishTest(BaseTest):
    writer_class = stardicter.czechenglish.CzechEnglishWriter


class CzechEnglishFileTest(CzechEnglishTest):
    def get_writer(self):
        '''
        Gets prepared writer class.
        '''
        return self.writer_class(
            file=io.open(os.path.join(
                os.path.dirname(__file__),
                'test_data.txt'
            ), 'rb')
        )



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Base module for stardciter convertors."""

from __future__ import unicode_literals

import codecs
import datetime
import gzip
import hashlib
import json
from operator import attrgetter
import os
import re
import struct
import time

from six.moves.urllib.request import urlopen
from six import BytesIO

from stardicter.word import Word


README_TEXT = r'''{title}
{line}

This is autogenerated dictionary for StarDict.

Data were downloaded from following website:
<{url}>

The original source is available under {license}.

Dictionary was generated using:
Stardicter version {version}

You can get conversion script from:
<https://cihar.com/software/slovnik/>

Install dictionary by copying dictionary files to dic/ folder in
StarDict. On Linux it is usually /usr/share/stardict/dic/, on Windows
C:\Program files\stardict\dic\.
'''

STRIPTAGS = re.compile(r"<.*?>", re.DOTALL)

CONFIGFILE = os.path.expanduser('~/.stardicter')

# type of word (used as title)
FMT_TYPE = '<span size="larger" color="darkred" weight="bold">{0}</span>\n'

AUTHOR = 'Stardicter'
URL = 'https://cihar.com/software/slovnik/'


class StardictWriter(object):
    '''
    Generic writer for stardict dictionary.
    '''
    url = None
    name = 'Generic'
    prefix = ''
    source = 'aa'
    target = 'bb'
    license = ''
    bidirectional = True
    download_url = None
    download_charset = 'utf-8'
    download_gzip = False

    def __init__(self, ascii=False, notags=False, keyprefix='',
                 source='', target='', file=None):
        self.words = {}
        self.reverse = {}
        self.description = ''
        self.ascii = ascii
        self.notags = notags
        self._data = None
        self._checksum = None
        self.keyprefix = keyprefix
        if source:
            self.source = source
        if target:
            self.target = target
        self.file = file

    @property
    def data(self):
        '''
        Returns downloaded data file.
        '''
        if self._data is None:
            self._data = self.download()
        return self._data

    @property
    def lines(self):
        '''
        Returns lines.
        '''
        return self.data.splitlines()

    @property
    def checksum(self):
        '''
        Returns data checksum.
        '''
        if self._checksum is None:
            self._checksum = self.get_checksum()
        return self._checksum

    def get_filename(self, forward=True):
        '''
        Returns filename for dictionary.
        '''
        if forward:
            name = '{0}-{1}'.format(self.source, self.target)
        else:
            name = '{0}-{1}'.format(self.target, self.source)

        suffix = ''
        if self.ascii:
            suffix += '-ascii'
        if self.notags:
            suffix += '-notags'

        return '{0}{1}{2}'.format(self.prefix, name, suffix)

    def get_name(self, forward=True):
        '''
        Returns dictionary name.
        '''
        return self.name

    def is_data_line(self, line):
        '''
        Checks whether line is used for checksum. Can be used to exclude
        timestamps from data.
        '''
        return True

    def is_header_line(self, line):
        '''
        Checks whether line is header.
        '''
        return line[0] == '#'

    def add_description(self, line):
        '''
        Adds description from line.
        '''
        self.description += line[1:] + '\n'

    def get_checksum(self):
        '''
        Calculated dictionary checksum.
        '''
        md5 = hashlib.md5()
        for line in self.lines:
            if self.is_data_line(line):
                md5.update(line.encode('utf-8'))
        return md5.hexdigest()

    def download(self):
        '''
        Downloads dictionary.
        '''
        if self.file:
            handle = self.file
            self.download_gzip = self.file.name.endswith('.gz')
        else:
            if self.download_url is None:
                return 'word\ttranslation\ttype\tnote\tauthor'
            handle = urlopen(self.download_url)
        if self.download_gzip:
            stringio = BytesIO(handle.read())
            handle.close()
            handle = gzip.GzipFile(fileobj=stringio)
        text = handle.read()
        return text.decode(self.download_charset)

    def parse_line(self, line):
        '''
        Parses single line with word.
        '''
        return [Word.from_slovnik(line)]

    def parse(self):
        '''
        Parses dictionary.
        '''
        for line in self.lines:
            # Skip blank lines
            if line.strip() == '':
                continue

            # Description from header
            if self.is_header_line(line):
                self.add_description(line)
                continue

            # Parse line
            words = self.parse_line(line)

            for word in words:

                # Skip not translated words
                if not word.word or not word.translation:
                    continue

                # Store word
                if len(word.word) < 256:
                    if word.word not in self.words:
                        self.words[word.word] = []

                    self.words[word.word].append(word)

                # Other direction
                if self.bidirectional and len(word.translation) < 256:
                    if word.translation not in self.reverse:
                        self.reverse[word.translation] = []
                    self.reverse[word.translation].append(word.reverse())

        # Sort by translation alphabetically

        for word in self.words:
            self.words[word].sort(key=attrgetter('translation'))

        for word in self.reverse:
            self.reverse[word].sort(key=attrgetter('translation'))

    def convert(self, text, convert=True):
        '''
        Converts text to match wanted format.
        '''
        if self.notags:
            text = STRIPTAGS.sub('', text)

        if not convert:
            return text

        if self.ascii:
            return text.encode('ascii', 'deaccent')

        return text.encode('utf-8')

    def formatentry(self, words):
        '''
        Formats dictionary entry.
        '''
        # sort alphabetically
        # array for different word types
        alltypes = [
            'n:',
            'v:',
            'adj:',
            'adv:',
            'prep:',
            'conj:',
            'interj:',
            'num:',
            '',
        ]
        # variables used for data
        result = ''
        typed = {}
        # array holding typed words
        for key in alltypes:
            typed[key] = []
        # process all translations
        for word in words:
            tokens = word.wtype.split()
            saved = False
            for key in alltypes:
                # check if translation is current type
                if key in tokens:
                    saved = True
                    # remove type from translation, it will be in title
                    del tokens[tokens.index(key)]
                    word.wtype = ' '.join(tokens)
                    # handle irregullar word specially (display them first)
                    if '[neprav.]' in tokens:
                        typed[key].insert(0, word)
                    else:
                        typed[key].append(word)
                    break
            if not saved:
                typed[''].append(word)

        # and finally convert entries to text
        for typ in alltypes:
            if typed[typ]:
                # header to display
                if typ == '':
                    result += '\n'
                else:
                    result += FMT_TYPE.format(typ)
                for word in typed[typ]:
                    result += '    '
                    result += word.format()

        return result

    def getsortedwords(self, words):
        '''
        Returns keys of hash sorted case insensitive.
        '''
        if self.ascii:
            tuples = [
                (item.encode('ascii', 'deaccent').lower(), item)
                for item in words
            ]
        else:
            tuples = [
                (item.encode('utf-8').lower(), item)
                for item in words
            ]
        tuples.sort()
        return [item[1] for item in tuples]

    def write_words(self, basefilename, name, words):
        '''
        Writes word list to dictionary files.
        '''
        # initialize variables
        offset = 0
        count = 0
        idxsize = 0

        # File names
        dictn = '{0}.dict'.format(basefilename)
        idxn = '{0}.idx'.format(basefilename)

        # Write dictionary and index
        with open(dictn, 'wb') as dictf, open(idxn, 'wb') as idxf:
            for key in self.getsortedwords(words):
                # format single entry
                entry = self.convert(self.formatentry(words[key]))

                # write dictionary text
                dictf.write(entry)

                # write index entry
                idxf.write(self.convert(key))
                idxf.write(b'\0')
                idxf.write(struct.pack(b'!I', offset))
                idxf.write(struct.pack(b'!I', len(entry)))

                # calculate offset for next index entry
                offset += len(entry)
                count += 1

            # index size is needed in ifo
            idxsize = idxf.tell()

        self._write_ifo(name, basefilename, count, idxsize)

    def _write_ifo(self, name, basefilename, count, idxsize):
        '''
        Writes info file.
        '''
        filename = '{0}.ifo'.format(basefilename)
        with codecs.open(filename, 'w', 'utf-8') as handle:
            handle.write('StarDict\'s dict ifo file\n')
            handle.write('version=2.4.2\n')
            handle.write(self.convert('bookname={0}\n'.format(name), False))
            handle.write('wordcount={0}\n'.format(count))
            handle.write('idxfilesize={0}\n'.format(idxsize))
            handle.write(self.convert('author={0}\n'.format(AUTHOR), False))
            handle.write(self.convert('website={0}\n'.format(URL), False))
            # we're using pango markup for all entries
            handle.write('sametypesequence=g\n')
            now = int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
            today = datetime.datetime.utcfromtimestamp(now)
            handle.write(today.strftime('date=%Y.%m.%d\n'))

    def write_dict(self, directory):
        '''
        Writes dictionary into directory.
        '''
        # Write readme
        with open(os.path.join(directory, 'README'), 'wb') as readme:
            readme.write(self.get_readme().encode('utf-8'))
            if self.description:
                readme.write(
                    '\nOriginal description of dictionary:\n{0}'.format(
                        self.description
                    ).encode('utf-8')
                )
        # Write forward dictioanry
        self.write_words(
            os.path.join(directory, self.get_filename(True)),
            self.get_name(True),
            self.words
        )
        # Write reverse dictionary
        if self.bidirectional:
            self.write_words(
                os.path.join(directory, self.get_filename(False)),
                self.get_name(False),
                self.reverse
            )

    def get_readme(self):
        '''
        Generates README text for dictionary.
        '''
        from stardicter import __version__
        title = '{0} for StarDict'.format(self.name)
        return README_TEXT.format(
            title=title,
            line='-' * len(title),
            url=self.url,
            license=self.license,
            version=__version__,
        )

    def get_config_key(self):
        '''
        Key used to store MD5 in config.
        '''
        return 'md5-{0}{1}'.format(self.keyprefix, self.get_filename())

    def load_config(self):
        '''
        Loads checksum cache.
        '''
        try:
            with open(CONFIGFILE) as handle:
                return json.load(handle)
        except (ValueError, IOError):
            return {}

    def save_config(self, changes):
        '''
        Loads checksum cache.
        '''
        config = self.load_config()
        config.update(changes)
        with codecs.open(CONFIGFILE, 'wb', 'utf-8') as handle:
            json.dump(config, handle, indent=2)

    def was_changed(self):
        '''
        Detects whether dictionary has same content as on last run.
        '''
        key = self.get_config_key()
        config = self.load_config()
        if key not in config:
            return True
        return self.checksum != config[key]

    def save_checksum(self):
        '''
        Saves checksum to configuration.
        '''
        key = self.get_config_key()
        self.save_config({key: self.checksum})

    def get_source_name(self):
        """Name for source file."""
        name = os.path.basename(self.download_url)
        if name.endswith('.gz'):
            name = name[:-3]
        return name

    def write_source(self, directory):
        """Write source file."""
        filename = os.path.join(directory, self.get_source_name())
        with open(filename, 'wb') as handle:
            handle.write(self.data.encode(self.download_charset))



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Convertor for dicts.info dictionaries"""

from __future__ import unicode_literals
from stardicter.base import StardictWriter
from stardicter.word import Word
from six.moves.urllib.request import urlopen

BASEURL = 'http://www.dicts.info/uddl.php?l1={0}&l2={1}&format=text'


class DictsInfoWriter(StardictWriter):
    url = 'http://www.dicts.info/'
    name = 'dicts.info'
    license = 'non redistributable license'

    def is_header_line(self, line):
        '''
        Checks whether line is header.
        '''
        return line.startswith('#')

    def add_description(self, line):
        '''
        Adds description from line.
        '''
        self.description += line[2:]

    def is_data_line(self, line):
        '''
        Checks whether line is used for checksum. Can be used to exclude
        timestamps from data.
        '''
        return 'created from the Universal dictionary at' not in line

    def parse_line(self, line):
        words, translations, wtype = line.split('\t')
        words = words.split(';')
        translations = translations.split(';')
        for word in words:
            for translation in translations:
                yield Word(word, translation, wtype=wtype)

    def download(self):
        '''
        Downloads dictionary data.
        '''
        handle = urlopen(
            BASEURL.format(self.source, self.target),
            'ok=selected'.encode('utf-8')
        )
        data = handle.read().decode('utf-8')

        if 'You cannot select two same languages.' in data:
            raise ValueError(data)

        if 'SQL select error' in data:
            raise ValueError(
                'Failed to fetch data, probably due to invalid language name.'
            )

        return data

    def get_name(self, forward=True):
        '''
        Returns dictionary name.
        '''
        template = 'dicts.info: {0}-{1}'
        if forward:
            return template.format(self.source, self.target)
        return template.format(self.target, self.source)



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Main executer for stardict convertor
"""

from __future__ import unicode_literals
from __future__ import print_function
from argparse import ArgumentParser
import io
import sys
import stardicter


def main():
    parser = ArgumentParser(
        description='Generates StarDict compatible dictionaries',
    )
    parser.add_argument(
        'dictionary',
        metavar='DICTIONARY',
        type=str,
        nargs='?',
        help='dictionary to download'
    )
    parser.add_argument(
        '-c',
        '--change',
        action='store_true',
        dest='change',
        default=False,
        help='Generate only on source data change.'
    )
    parser.add_argument(
        '-A',
        '--all',
        action='store_true',
        dest='all',
        default=False,
        help='Generate fomat combinations.'
    )
    parser.add_argument(
        '-a',
        '--ascii',
        action='store_true',
        dest='ascii',
        default=False,
        help='Generate plain ascii dictionary.'
    )
    parser.add_argument(
        '-S',
        '--write-source',
        action='store_true',
        dest='write_source',
        default=False,
        help='Generate source tarball.',
    )
    parser.add_argument(
        '-n',
        '--notags',
        action='store_true',
        dest='notags',
        default=False,
        help='Generate dictionary without pango markup.'
    )
    parser.add_argument(
        '-l',
        '--list',
        action='store_true',
        dest='list',
        default=False,
        help='Lists available dictionaries.'
    )
    parser.add_argument(
        '-d',
        '--directory',
        dest='directory',
        default='.',
        help='Directory where to store generated dictionaries'
    )
    parser.add_argument(
        '-f',
        '--file',
        dest='file',
        default=None,
        type=lambda name: io.open(name, 'rb'),
        help='File to use for reading the data instead of default download.',
    )
    parser.add_argument(
        '-s',
        '--source',
        dest='source',
        default='',
        help='Source language for multilanguage dictionaries'
    )
    parser.add_argument(
        '-t',
        '--target',
        dest='target',
        default='',
        help='Target language for multilanguage dictionaries'
    )
    parser.add_argument(
        '-m',
        '--monthly',
        action='store_true',
        dest='monthly',
        default=False,
        help='Flag indicating montly runs (for checksum checking)'
    )

    options = parser.parse_args()

    if options.list:
        for name in stardicter.DICTIONARIES:
            obj = stardicter.DICTIONARIES[name]
            print(
                '{0}: {1} <{2}>'.format(
                    name, obj.name, obj.url
                )
            )
        return

    if options.dictionary is None:
        print('You have to specify dictionary to process!')
        parser.print_usage()
        sys.exit(1)

    if options.dictionary not in stardicter.DICTIONARIES:
        print('Unknown dictionary, use -l to list available ones.')
        parser.print_usage()
        sys.exit(1)

    keyprefix = ''
    if options.monthly:
        keyprefix = 'monthly-'

    writer = stardicter.DICTIONARIES[options.dictionary](
        keyprefix=keyprefix,
        source=options.source,
        target=options.target,
        file=options.file
    )

    # Change detection
    if options.change and not writer.was_changed():
        sys.exit(0)

    # Load data
    writer.parse()

    # Generate source package
    if options.write_source:
        writer.write_source(options.directory)

    # Write dictionaries
    if options.all:
        params = (
            (True, True),
            (True, False),
            (False, True),
            (False, False),
        )
    else:
        params = ((options.ascii, options.notags),)

    for param in params:
        writer.ascii = param[0]
        writer.notags = param[1]
        writer.write_dict(options.directory)

    # Change detection
    if options.change:
        writer.save_checksum()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright © 2015 Michal Čihař <michal@cihar.com>
#
# This file is part of Odorik <https://github.com/nijel/stardicter>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Setup file for easy installation."""
import io
import os
from setuptools import setup

VERSION = __import__('stardicter').__version__
README = os.path.join(os.path.dirname(__file__), 'README.rst')

with io.open(README, encoding='utf-8') as readme:
    LONG_DESCRIPTION = readme.read()

REQUIRES = open('requirements.txt').read().split()

setup(
    name='stardicter',
    version=VERSION,
    author='Michal Čihař',
    author_email='michal@cihar.com',
    description=' Conversion tools from various formats to StarDict.',
    license='GPLv3+',
    keywords='stardict,dictionary',
    url='https://cihar.com/software/slovnik/',
    download_url='https://pypi.python.org/pypi/stardicter',
    bugtrack_url='https://github.com/nijel/stardicter/issues',
    platforms=['any'],
    packages=[
        'stardicter',
    ],
    package_data={'stardicter': ['test_*.txt']},
    package_dir={'stardicter': 'stardicter'},
    long_description=LONG_DESCRIPTION,
    install_requires=REQUIRES,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    entry_points={
        'console_scripts': ['sdgen = stardicter.main:main']
    },
)



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Test for word handling"""

import unittest
from stardicter.word import Word


class WordTest(unittest.TestCase):
    '''
    Word class testsing.
    '''
    def do_parse(self, line, expected, translation):
        '''
        Test for word parsing.
        '''
        word = Word.from_slovnik(line)
        self.assertEqual(word.word, expected)
        self.assertEquals(word.translation, translation)
        return word

    def test_parse(self):
        '''
        Testing various weird stuff in parser.
        '''
        word = self.do_parse(
            'a\tb\ttype\tnote\tauthor',
            'a', 'b'
        )
        self.assertEquals(word.wtype, 'type')
        self.assertEquals(word.note, 'note')
        self.assertEquals(word.author, 'author')

    def test_fixups(self):
        '''
        Test for parsing fixups.
        '''
        self.do_parse(
            'a\tc\tb\ttype\tnote\tauthor',
            'ac', 'b'
        )
        self.do_parse(
            'a\tb\ttype\tnote',
            'a', 'b'
        )
        self.do_parse(
            'a\tb\ttype',
            'a', 'b'
        )
        self.do_parse(
            'a\tb',
            'a', 'b'
        )
        self.do_parse(
            'a',
            'a', ''
        )

    def test_no_fixup(self):
        '''
        Test for not detected fixup.
        '''
        self.assertRaisesRegexp(
            ValueError,
            r'Invalid input: \'\\t\\t\\t\\t\\t\\t\\t\\t\'',
            self.do_parse,
            '\t\t\t\t\t\t\t\t',
            '', ''
        )



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Covertor for GNU/FDL Anglicko-Český slovník"""

from __future__ import unicode_literals
from stardicter.base import StardictWriter

URL = 'https://www.svobodneslovniky.cz/data/en-cs.txt.gz'


class CzechEnglishWriter(StardictWriter):
    url = 'https://www.svobodneslovniky.cz/'
    name = 'GNU/FDL Anglicko-Český slovník'
    source = 'english'
    target = 'czech'
    license = 'GFDL-1.1'
    download_url = URL
    download_gzip = True

    def is_data_line(self, line):
        '''
        Checks whether this is line with timestamp.
        '''
        return not line.startswith('#      Date:')

    def get_name(self, forward=True):
        if forward:
            return 'GNU/FDL Anglicko-Český slovník'
        return 'GNU/FDL Česko-Anglický slovník'



# -*- coding: utf-8 -*-
#
# Copyright © 2006 - 2017 Michal Čihař <michal@cihar.com>
#
# This file is part of Stardicter <https://cihar.com/software/slovnik/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Test for accent removal"""

from __future__ import unicode_literals
import unittest


class DeaccentTest(unittest.TestCase):
    def test_deaccent(self):
        self.assertEqual(
            'zkouška'.encode('ascii', 'deaccent'),
            b'zkouska'
        )

    def test_quotes(self):
        self.assertEqual(
            '\x93\x94\x84\x92'.encode('ascii', 'deaccent'),
            b'"""\''
        )
